#include <stdio.h>
#include <avs/flow.h>
#include <avs/avs_data.h>
#include <avs/field.h>
#include <avs/geom.h>

AVSinit_modules()
{
	int polygon_to_geom();

	AVSmodule_from_desc(polygon_to_geom);
}

polygon_to_geom()
{
	int polygon_compute();	/* declare the compute function (below) */
	int out_port;	/* temporaries to hold the port numbers */
	int parm, rarm;

	AVSset_module_name("Plot Poly", MODULE_DATA);

	out_port = AVScreate_output_port(/*name*/ "Geometry", /*type*/"geom");

	parm = AVSadd_parameter("Poly filename", "string", NULL, NULL, NULL);
   	AVSconnect_widget(parm,"browser");

	rarm = AVSadd_float_parameter("Node Size", 0.0001, 0.0, 1000.0);
	AVSconnect_widget(rarm,"typein_real");

	/* Tell avs what subroutine to call to do the compute */
	AVSset_compute_proc(polygon_compute);
}

#define MAXVERTS		10
#define MAXCOLOR		8

polygon_compute(output, filename, ptrradius)
GEOMedit_list *output;
char *filename;
float *ptrradius;
{
	GEOMobj *faces[MAXCOLOR], *edges[MAXCOLOR], *nodes[MAXCOLOR];
	int faceobj[MAXCOLOR], edgeobj[MAXCOLOR], nodeobj[MAXCOLOR];
	FILE *fp;
	int i, nvert, ipoint, icolor, npoint[MAXCOLOR];
	float *radii[MAXCOLOR], *r[MAXCOLOR];
	float verts[MAXVERTS][3], col[MAXVERTS][3];
	char name[16];

	if (filename == NULL) return(0); 
	if ((fp = fopen(filename,"r")) == NULL) return(0); 

	for(icolor=0; icolor<MAXCOLOR; icolor++)
		faceobj[icolor] = edgeobj[icolor] = nodeobj[icolor] = npoint[icolor] = 0;

	while(!feof(fp)){
		fscanf(fp, "%d %d", &icolor, &nvert);
		for(i=0; i<nvert; i++)
			fscanf(fp, "%f %f %f", &verts[i][0], &verts[i][1], &verts[i][2]);

		switch(nvert){
			case 1:
				nodeobj[icolor]++;
				break;
			case 2:
				edgeobj[icolor]++;
				break;
			default:
				faceobj[icolor]++;
		}
	}

	for(icolor=0; icolor<MAXCOLOR; icolor++){
		if(faceobj[icolor]) faces[icolor] = GEOMcreate_obj(GEOM_POLYHEDRON, NULL);
		if(edgeobj[icolor]) edges[icolor] = GEOMcreate_obj(GEOM_POLYTRI, NULL);
		if(nodeobj[icolor]) {
			radii[icolor] = (float *)malloc(nodeobj[icolor]*sizeof(float));
			r[icolor] = (float *)malloc(nodeobj[icolor]*3*sizeof(float));
		}
	}

	rewind(fp);
	while(!feof(fp)){
		fscanf(fp, "%d %d", &icolor, &nvert);
		for(i=0; i<nvert; i++)
			fscanf(fp, "%f %f %f", &verts[i][0], &verts[i][1], &verts[i][2]);

		switch(nvert){
			case 1:
				memcpy(r[icolor] + 3*npoint[icolor], verts, 3*sizeof(float));
				radii[icolor][npoint[icolor]] = *ptrradius;
				npoint[icolor]++;
				break;
			case 2:
				GEOMadd_disjoint_line(edges[icolor],verts,NULL,2,0);
				break;
			default:
				GEOMadd_disjoint_polygon(faces[icolor],verts,NULL,NULL,nvert,GEOM_SHARED,0);
		}
	}

	for(icolor=0; icolor<MAXCOLOR; icolor++)
		if(faceobj[icolor]) GEOMgen_normals(faces[icolor], GEOM_FACET_NORMALS);

	for(icolor=0; icolor<MAXCOLOR; icolor++)
		if(nodeobj[icolor]) 
			nodes[icolor] = GEOMcreate_sphere(NULL, r[icolor], radii[icolor], NULL, NULL, nodeobj[icolor], 0);

	*output = GEOMinit_edit_list(*output);
	for(icolor=0; icolor<MAXCOLOR; icolor++){
		if(faceobj[icolor]){
			sprintf(name, "faces%d", icolor);
			GEOMedit_geometry(*output,name,faces[icolor]);
		}
		if(edgeobj[icolor]){
			sprintf(name, "edges%d", icolor);
			GEOMedit_geometry(*output,name,edges[icolor]);
		}
		if(nodeobj[icolor]){
			sprintf(name, "nodes%d", icolor);
			GEOMedit_geometry(*output,name,nodes[icolor]);
		}
	}

	for(icolor=0; icolor<MAXCOLOR; icolor++){
		if(faceobj[icolor]) GEOMdestroy_obj(faces[icolor]); 
		if(edgeobj[icolor]) GEOMdestroy_obj(edges[icolor]); 
		if(nodeobj[icolor]) GEOMdestroy_obj(nodes[icolor]); 
	}

	return(1);
}
